import React, { useState, useEffect } from 'react';
import { useNavigate } from 'react-router-dom';
import { useDispatch, useSelector } from 'react-redux';
import { showNotification } from '../../store/notification.slice';
import { fetchAvailableDatasets, fetchChats, createChatRecord } from '../../store/chat/chat.thunks';
import SvgIcon from '../../components/SvgIcon';

export default function NewChat() {
    const navigate = useNavigate();
    const dispatch = useDispatch();
    const [selectedDatasetId, setSelectedDatasetId] = useState(null);
    const [isNavigating, setIsNavigating] = useState(false);

    // 从 Redux store 获取可用知识库数据
    const datasets = useSelector((state) => state.chat.availableDatasets.items || []);
    const isLoading = useSelector((state) => state.chat.availableDatasets.status === 'loading');
    const error = useSelector((state) => state.chat.availableDatasets.error);

    // 获取聊天历史记录
    const chatHistory = useSelector((state) => state.chat.history.items || []);
    const chatHistoryLoading = useSelector((state) => state.chat.history.status === 'loading');
    const chatCreationStatus = useSelector((state) => state.chat.sendMessage?.status);

    // 获取可用知识库列表和聊天历史
    useEffect(() => {
        dispatch(fetchAvailableDatasets());
        dispatch(fetchChats({ page: 1, page_size: 50 }));
    }, [dispatch]);

    // 监听错误状态
    useEffect(() => {
        if (error) {
            dispatch(
                showNotification({
                    message: `获取可用知识库列表失败: ${error}`,
                    type: 'danger',
                })
            );
        }
    }, [error, dispatch]);

    // 处理知识库选择
    const handleSelectKnowledgeBase = async (dataset) => {
        if (isNavigating) return; // 如果正在导航中，阻止重复点击

        try {
            setSelectedDatasetId(dataset.id);
            setIsNavigating(true);

            // 判断聊天历史中是否已存在该知识库的聊天记录
            const existingChat = chatHistory.find((chat) => {
                // 检查知识库ID是否匹配
                if (chat.datasets && Array.isArray(chat.datasets)) {
                    return chat.datasets.some((ds) => ds.id === dataset.id);
                }
                // 兼容旧格式
                if (chat.dataset_id_list && Array.isArray(chat.dataset_id_list)) {
                    return chat.dataset_id_list.includes(dataset.id.replace(/-/g, ''));
                }
                return false;
            });

            if (existingChat) {
                // 找到现有聊天记录，导航到该聊天页面
                console.log(`找到现有聊天记录，直接导航到 /chat/${dataset.id}/${existingChat.conversation_id}`);
                navigate(`/chat/${dataset.id}/${existingChat.conversation_id}`);
            } else {
                // 没有找到现有聊天记录，直接创建新的聊天（而不是导航到 /chat/${dataset.id}）
                console.log(`未找到现有聊天记录，直接创建新的聊天，知识库ID: ${dataset.id}`);

                // 直接在这里创建聊天记录，避免在 Chat.jsx 中再次创建
                const response = await dispatch(
                    createChatRecord({
                        dataset_id_list: [dataset.id.replace(/-/g, '')],
                        question: '选择当前知识库，创建聊天',
                    })
                ).unwrap();

                if (response && response.conversation_id) {
                    console.log(`创建成功，导航到 /chat/${dataset.id}/${response.conversation_id}`);
                    navigate(`/chat/${dataset.id}/${response.conversation_id}`);
                } else {
                    throw new Error('未能获取会话ID');
                }
            }
        } catch (error) {
            console.error('导航或创建聊天失败:', error);
            dispatch(
                showNotification({
                    message: `创建聊天失败: ${error.message || '请重试'}`,
                    type: 'danger',
                })
            );
            setIsNavigating(false);
            setSelectedDatasetId(null);
        }
    };

    // 渲染加载状态
    if (isLoading || chatHistoryLoading) {
        return (
            <div className='container-fluid px-4 py-5 text-center'>
                <div className='spinner-border' role='status'>
                    <span className='visually-hidden'>加载中...</span>
                </div>
            </div>
        );
    }

    return (
        <div className='container-fluid px-4 py-5'>
            {/* 导航中的遮罩层 */}
            {isNavigating && (
                <div
                    className='position-fixed top-0 start-0 w-100 h-100 d-flex justify-content-center align-items-center'
                    style={{
                        backgroundColor: 'rgba(255, 255, 255, 0.7)',
                        zIndex: 1050,
                    }}
                >
                    <div className='text-center'>
                        <div className='spinner-border mb-2' role='status'>
                            <span className='visually-hidden'>加载中...</span>
                        </div>
                        <div>正在加载聊天界面...</div>
                    </div>
                </div>
            )}

            <h4 className='mb-4'>选择知识库开始聊天</h4>
            <div className='row row-cols-1 row-cols-md-2 row-cols-lg-3 g-4 justify-content-center'>
                {datasets.length > 0 ? (
                    datasets.map((dataset) => (
                        <div key={dataset.id} className='col'>
                            <div
                                className={`card h-100 shadow-sm border-0 ${!isNavigating ? 'cursor-pointer' : ''} ${
                                    selectedDatasetId === dataset.id ? 'border-primary' : ''
                                }`}
                                onClick={() => handleSelectKnowledgeBase(dataset)}
                                style={{ opacity: isNavigating && selectedDatasetId !== dataset.id ? 0.6 : 1 }}
                            >
                                <div className='card-body'>
                                    <h5 className='card-title d-flex justify-content-between align-items-center'>
                                        {dataset.name}
                                        {selectedDatasetId === dataset.id && isNavigating && (
                                            <div
                                                className='spinner-border spinner-border-sm text-primary'
                                                role='status'
                                            >
                                                <span className='visually-hidden'>加载中...</span>
                                            </div>
                                        )}
                                    </h5>
                                    <p className='card-text text-muted'>{dataset.desc || dataset.description || ''}</p>
                                    <div className='text-muted small d-flex align-items-center gap-2'>
                                        <span className='d-flex align-items-center gap-1'>
                                            <SvgIcon className='file' />
                                            {dataset.document_count || 0} 文档
                                        </span>
                                        <span className='d-flex align-items-center gap-1'>
                                            <SvgIcon className='clock' />
                                            {dataset.create_time
                                                ? new Date(dataset.create_time).toLocaleDateString()
                                                : 'N/A'}
                                        </span>
                                    </div>
                                </div>
                            </div>
                        </div>
                    ))
                ) : (
                    <div className='col-12'>
                        <div className='alert alert-warning'>暂无可访问的知识库，请先申请知识库访问权限</div>
                    </div>
                )}
            </div>
        </div>
    );
}
