import React, { useState, useEffect, useRef } from 'react';
import { useDispatch } from 'react-redux';
import { showNotification } from '../../../store/notification.slice';
import SvgIcon from '../../../components/SvgIcon';

// 导入拆分的组件
import Breadcrumb from './components/Breadcrumb';
import DocumentList from './components/DocumentList';
import FileUploadModal from './components/FileUploadModal';

export default function DatasetTab({ knowledgeBase }) {
    const dispatch = useDispatch();
    const [searchQuery, setSearchQuery] = useState('');
    const [selectedDocuments, setSelectedDocuments] = useState([]);
    const [selectAll, setSelectAll] = useState(false);
    const [showBatchDropdown, setShowBatchDropdown] = useState(false);
    const [showAddFileModal, setShowAddFileModal] = useState(false);
    const [isSubmitting, setIsSubmitting] = useState(false);
    const [newFile, setNewFile] = useState({
        name: '',
        description: '',
        file: null,
    });
    const [fileErrors, setFileErrors] = useState({});
    const dropdownRef = useRef(null);
    const fileInputRef = useRef(null);

    // Use documents from knowledge base or empty array if not available
    const [documents, setDocuments] = useState(knowledgeBase.documents || []);

    // Update documents when knowledgeBase changes
    useEffect(() => {
        setDocuments(knowledgeBase.documents || []);
    }, [knowledgeBase]);

    // Handle click outside dropdown
    useEffect(() => {
        function handleClickOutside(event) {
            if (dropdownRef.current && !dropdownRef.current.contains(event.target)) {
                setShowBatchDropdown(false);
            }
        }

        // Add event listener
        document.addEventListener('mousedown', handleClickOutside);
        return () => {
            // Remove event listener on cleanup
            document.removeEventListener('mousedown', handleClickOutside);
        };
    }, [dropdownRef]);

    // Handle search input change
    const handleSearchChange = (e) => {
        setSearchQuery(e.target.value);
    };

    // Handle select all checkbox
    const handleSelectAll = () => {
        if (selectAll) {
            setSelectedDocuments([]);
        } else {
            setSelectedDocuments(documents.map((doc) => doc.id));
        }
        setSelectAll(!selectAll);
    };

    // Handle individual document selection
    const handleSelectDocument = (docId) => {
        if (selectedDocuments.includes(docId)) {
            setSelectedDocuments(selectedDocuments.filter((id) => id !== docId));
            setSelectAll(false);
        } else {
            setSelectedDocuments([...selectedDocuments, docId]);
            if (selectedDocuments.length + 1 === documents.length) {
                setSelectAll(true);
            }
        }
    };

    // Handle batch delete
    const handleBatchDelete = () => {
        if (selectedDocuments.length === 0) return;

        // Here you would typically call an API to delete the selected documents
        console.log('Deleting documents:', selectedDocuments);

        // Update documents state by removing selected documents
        setDocuments((prevDocuments) => prevDocuments.filter((doc) => !selectedDocuments.includes(doc.id)));

        // Show notification
        dispatch(
            showNotification({
                message: '已删除选中的数据集',
                type: 'success',
            })
        );

        // Reset selection
        setSelectedDocuments([]);
        setSelectAll(false);
        setShowBatchDropdown(false);
    };

    // Handle file input change
    const handleFileChange = (e) => {
        const selectedFile = e.target.files[0];
        if (selectedFile) {
            setNewFile({
                ...newFile,
                name: selectedFile.name,
                file: selectedFile,
            });

            // Clear file error if exists
            if (fileErrors.file) {
                setFileErrors((prev) => ({
                    ...prev,
                    file: '',
                }));
            }
        }
    };

    // Handle description input change
    const handleDescriptionChange = (e) => {
        setNewFile({
            ...newFile,
            description: e.target.value,
        });
    };

    // Handle file drop
    const handleFileDrop = (e) => {
        e.preventDefault();
        e.stopPropagation();

        const droppedFile = e.dataTransfer.files[0];
        if (droppedFile) {
            setNewFile({
                ...newFile,
                name: droppedFile.name,
                file: droppedFile,
            });

            // Clear file error if exists
            if (fileErrors.file) {
                setFileErrors((prev) => ({
                    ...prev,
                    file: '',
                }));
            }
        }
    };

    // Prevent default behavior for drag events
    const handleDragOver = (e) => {
        e.preventDefault();
        e.stopPropagation();
    };

    // Validate file form
    const validateFileForm = () => {
        const errors = {};

        if (!newFile.file) {
            errors.file = '请上传文件';
        }

        setFileErrors(errors);
        return Object.keys(errors).length === 0;
    };

    // Handle file upload
    const handleFileUpload = () => {
        // Validate form
        if (!validateFileForm()) {
            return;
        }

        setIsSubmitting(true);

        // Here you would typically call an API to upload the file
        console.log('Uploading file:', newFile);

        // Simulate API call
        setTimeout(() => {
            // Generate a new ID for the document
            const newId = Date.now().toString();

            // Format file size
            const fileSizeKB = newFile.file ? (newFile.file.size / 1024).toFixed(0) + 'kb' : '0kb';

            // Get current date
            const today = new Date();
            const formattedDate = today.toISOString();

            // Create new document object
            const newDocument = {
                id: newId,
                name: newFile.name,
                description: newFile.description || '无描述',
                size: fileSizeKB,
                create_time: formattedDate,
                update_time: formattedDate,
            };

            // Add new document to the documents array
            setDocuments((prevDocuments) => [...prevDocuments, newDocument]);

            // Show notification
            dispatch(
                showNotification({
                    message: '数据集上传成功',
                    type: 'success',
                })
            );

            setIsSubmitting(false);
            // Reset form and close modal
            handleCloseAddFileModal();
        }, 1000);
    };

    // Open file selector when clicking on the upload area
    const handleUploadAreaClick = () => {
        fileInputRef.current.click();
    };

    // Handle close modal
    const handleCloseAddFileModal = () => {
        setNewFile({
            name: '',
            description: '',
            file: null,
        });
        setFileErrors({});
        setShowAddFileModal(false);
    };

    // Handle delete document
    const handleDeleteDocument = (docId) => {
        // Here you would typically call an API to delete the document
        console.log('Deleting document:', docId);

        // Update documents state by removing the deleted document
        setDocuments((prevDocuments) => prevDocuments.filter((doc) => doc.id !== docId));

        // Show notification
        dispatch(
            showNotification({
                message: '数据集已删除',
                type: 'success',
            })
        );
    };

    // Filter documents based on search query
    const filteredDocuments = documents.filter(
        (doc) =>
            doc.name.toLowerCase().includes(searchQuery.toLowerCase()) ||
            (doc.description && doc.description.toLowerCase().includes(searchQuery.toLowerCase()))
    );

    return (
        <>
            {/* Breadcrumb navigation */}
            <Breadcrumb knowledgeBase={knowledgeBase} activeTab='datasets' />

            {/* Toolbar */}
            <div className='d-flex justify-content-between align-items-center mb-3'>
                <div className='d-flex gap-2'>
                    <button
                        className='btn btn-dark d-flex align-items-center gap-1'
                        onClick={() => setShowAddFileModal(true)}
                    >
                        <SvgIcon className='plus' />
                        上传文件
                    </button>
                    {selectedDocuments.length > 0 && (
                        <div className='dropdown' ref={dropdownRef}>
                            <button
                                className='btn btn-outline-secondary dropdown-toggle'
                                type='button'
                                onClick={() => setShowBatchDropdown(!showBatchDropdown)}
                            >
                                批量操作 ({selectedDocuments.length})
                            </button>
                            {showBatchDropdown && (
                                <ul
                                    className='dropdown-menu show'
                                    style={{
                                        position: 'absolute',
                                        inset: '0px auto auto 0px',
                                        margin: '0px',
                                        transform: 'translate(0px, 40px)',
                                    }}
                                >
                                    <li>
                                        <button className='dropdown-item text-danger' onClick={handleBatchDelete}>
                                            <SvgIcon className='trash' />
                                            <span className='ms-1'>删除所选</span>
                                        </button>
                                    </li>
                                </ul>
                            )}
                        </div>
                    )}
                </div>
                <div className='w-25'>
                    <input
                        type='text'
                        className='form-control'
                        placeholder='搜索数据集...'
                        value={searchQuery}
                        onChange={handleSearchChange}
                    />
                </div>
            </div>

            {/* Document list */}
            <DocumentList
                documents={filteredDocuments}
                selectedDocuments={selectedDocuments}
                selectAll={selectAll}
                onSelectAll={handleSelectAll}
                onSelectDocument={handleSelectDocument}
                onDeleteDocument={handleDeleteDocument}
            />
            {/* Pagination */}
            <div className='d-flex justify-content-between align-items-center mt-3'>
                <div>
                    每页行数:
                    <select className='form-select form-select d-inline-block ms-2' style={{ width: '70px' }}>
                        <option value='5'>5</option>
                        <option value='10'>10</option>
                        <option value='20'>20</option>
                    </select>
                </div>
                <div className='d-flex align-items-center'>
                    <span className='me-3'>1-5 of 10</span>
                    <nav aria-label='Page navigation'>
                        <ul className='pagination pagination mb-0'>
                            <li className='page-item'>
                                <button className='page-link' aria-label='Previous'>
                                    <span aria-hidden='true'>&laquo;</span>
                                </button>
                            </li>
                            <li className='page-item'>
                                <button className='page-link' aria-label='Next'>
                                    <span aria-hidden='true'>&raquo;</span>
                                </button>
                            </li>
                        </ul>
                    </nav>
                </div>
            </div>
            
            {/* File upload modal */}
            <FileUploadModal
                show={showAddFileModal}
                newFile={newFile}
                fileErrors={fileErrors}
                isSubmitting={isSubmitting}
                onClose={handleCloseAddFileModal}
                onDescriptionChange={handleDescriptionChange}
                onFileChange={handleFileChange}
                onFileDrop={handleFileDrop}
                onDragOver={handleDragOver}
                onUploadAreaClick={handleUploadAreaClick}
                onUpload={handleFileUpload}
            />
        </>
    );
}
