import axios from 'axios';
import CryptoJS from 'crypto-js';
import { mockGet, mockPost, mockPut, mockDelete } from './mockApi';

const secretKey = import.meta.env.VITE_SECRETKEY;

// API连接状态
let isServerDown = false;
let hasCheckedServer = false;

// Create Axios instance with base URL
const api = axios.create({
    baseURL: '/api',
    withCredentials: true, // Include cookies if needed
});

// Request Interceptor
api.interceptors.request.use(
    (config) => {
        const encryptedToken = sessionStorage.getItem('token') || '';
        if (encryptedToken) {
            const decryptedToken = CryptoJS.AES.decrypt(encryptedToken, secretKey).toString(CryptoJS.enc.Utf8);
            config.headers.Authorization = `Token ${decryptedToken}`;
        }
        return config;
    },
    (error) => {
        console.error('Request error:', error);
        return Promise.reject(error);
    }
);

// Response Interceptor
api.interceptors.response.use(
    (response) => {
        // 如果成功收到响应，表示服务器正常工作
        if (!hasCheckedServer) {
            console.log('Server is up and running');
            isServerDown = false;
            hasCheckedServer = true;
        }
        return response;
    },
    (error) => {
        // 处理服务器无法连接的情况
        if (!error.response || error.code === 'ECONNABORTED' || error.message.includes('Network Error')) {
            console.error('Server appears to be down. Switching to mock data.');
            isServerDown = true;
            hasCheckedServer = true;
        }

        // Handle errors in the response
        if (error.response) {
            // monitor /verify
            if (error.response.status === 401 && error.config.url === '/auth/verify-token/') {
                if (window.location.pathname !== '/login' && window.location.pathname !== '/signup') {
                    window.location.href = '/login';
                }
            }

            // The request was made and the server responded with a status code
            console.error('API Error Response:', error.response.status, error.response.data.message);
            // alert(`Error: ${error.response.status} - ${error.response.data.message || 'Something went wrong'}`);
        } else if (error.request) {
            // The request was made but no response was received
            console.error('API Error Request:', error.request);
            // alert('Network error: No response from server');
        } else {
            // Something happened in setting up the request
            console.error('API Error Message:', error.message);
            // alert('Error: ' + error.message);
        }
        return Promise.reject(error); // Reject the promise
    }
);

// 检查服务器状态
export const checkServerStatus = async () => {
    try {
        // await api.get('/health-check', { timeout: 3000 });
        isServerDown = false;
        hasCheckedServer = true;
        console.log('Server connection established');
        return true;
    } catch (error) {
        isServerDown = true;
        hasCheckedServer = true;
        console.error('Server connection failed, using mock data');
        return false;
    }
};

// 初始检查服务器状态
checkServerStatus();

// Define common HTTP methods with fallback to mock API
const get = async (url, params = {}) => {
    try {
        if (isServerDown) {
            console.log(`[MOCK MODE] GET ${url}`);
            return await mockGet(url, params);
        }
        
        const res = await api.get(url, { ...params });
        return res.data;
    } catch (error) {
        if (!hasCheckedServer || (error.request && !error.response)) {
            console.log(`Failed to connect to server. Falling back to mock API for GET ${url}`);
            return await mockGet(url, params);
        }
        throw error;
    }
};

// Handle POST requests for JSON data with fallback to mock API
const post = async (url, data, isMultipart = false) => {
    try {
        if (isServerDown) {
            console.log(`[MOCK MODE] POST ${url}`);
            return await mockPost(url, data);
        }

        const headers = isMultipart
            ? { 'Content-Type': 'multipart/form-data' } // For file uploads
            : { 'Content-Type': 'application/json' }; // For JSON data

        const res = await api.post(url, data, { headers });
        return res.data;
    } catch (error) {
        if (!hasCheckedServer || (error.request && !error.response)) {
            console.log(`Failed to connect to server. Falling back to mock API for POST ${url}`);
            return await mockPost(url, data);
        }
        throw error;
    }
};

// Handle PUT requests with fallback to mock API
const put = async (url, data) => {
    try {
        if (isServerDown) {
            console.log(`[MOCK MODE] PUT ${url}`);
            return await mockPut(url, data);
        }

        const res = await api.put(url, data, {
            headers: { 'Content-Type': 'application/json' },
        });
        return res.data;
    } catch (error) {
        if (!hasCheckedServer || (error.request && !error.response)) {
            console.log(`Failed to connect to server. Falling back to mock API for PUT ${url}`);
            return await mockPut(url, data);
        }
        throw error;
    }
};

// Handle DELETE requests with fallback to mock API
const del = async (url) => {
    try {
        if (isServerDown) {
            console.log(`[MOCK MODE] DELETE ${url}`);
            return await mockDelete(url);
        }

        const res = await api.delete(url);
        return res.data;
    } catch (error) {
        if (!hasCheckedServer || (error.request && !error.response)) {
            console.log(`Failed to connect to server. Falling back to mock API for DELETE ${url}`);
            return await mockDelete(url);
        }
        throw error;
    }
};

const upload = async (url, data) => {
    try {
        if (isServerDown) {
            console.log(`[MOCK MODE] Upload ${url}`);
            return await mockPost(url, data, true);
        }

        const axiosInstance = await axios.create({
            baseURL: '/api',
            headers: {
                'Content-Type': 'multipart/form-data',
            },
        });
        const res = await axiosInstance.post(url, data);
        return res.data;
    } catch (error) {
        if (!hasCheckedServer || (error.request && !error.response)) {
            console.log(`Failed to connect to server. Falling back to mock API for Upload ${url}`);
            return await mockPost(url, data, true);
        }
        throw error;
    }
};

// 手动切换到模拟API（为调试目的）
export const switchToMockApi = () => {
    isServerDown = true;
    hasCheckedServer = true;
    console.log('Manually switched to mock API');
};

// 手动切换回真实API
export const switchToRealApi = async () => {
    // 重新检查服务器状态
    const isServerUp = await checkServerStatus();
    console.log(isServerUp ? 'Switched back to real API' : 'Server still down, continuing with mock API');
    return isServerUp;
};

// 权限相关API
export const applyPermission = (data) => {
    return post('/permissions/', data);
};

export const updatePermission = (data) => {
    return post('/permissions/update_permission/', data);
};

export const approvePermission = (permissionId) => {
    return post(`/permissions/approve_permission/${permissionId}`);
};

export const rejectPermission = (permissionId) => {
    return post(`/permissions/reject_permission/${permissionId}`);
};

export { get, post, put, del, upload };
